@@ -3,7 +3,7 @@ ZeroMQ connection.
 """
 from collections import deque, namedtuple
 
-from zmq.core import constants, error
+from zmq.core import constants, error, version
 from zmq.core.socket import Socket
 
 from zope.interface import implements
@@ -12,6 +12,10 @@ from twisted.internet.interfaces import IFileDescriptor, IReadDescriptor
 from twisted.python import log
 
 
+# Patch zmq.core.constants to support both zeromq2 and zeromq3
+ZMQ3 = version.zmq_version_info()[0] >= 3
+
+
 class ZmqEndpointType(object):
     """
     Endpoint could be "bound" or "connected".
@@ -77,10 +81,19 @@ class ZmqConnection(object):
 
         self.fd = self.socket.getsockopt(constants.FD)
         self.socket.setsockopt(constants.LINGER, factory.lingerPeriod)
-        self.socket.setsockopt(
-            constants.MCAST_LOOP, int(self.allowLoopbackMulticast))
+
+        if not ZMQ3:
+            self.socket.setsockopt(
+                constants.MCAST_LOOP, int(self.allowLoopbackMulticast))
+
         self.socket.setsockopt(constants.RATE, self.multicastRate)
-        self.socket.setsockopt(constants.HWM, self.highWaterMark)
+
+        if not ZMQ3:
+            self.socket.setsockopt(constants.HWM, self.highWaterMark)
+        else:
+            self.socket.setsockopt(constants.SNDHWM, self.highWaterMark)
+            self.socket.setsockopt(constants.RCVHWM, self.highWaterMark)
+
         if self.identity is not None:
             self.socket.setsockopt(constants.IDENTITY, self.identity)
 
@@ -169,21 +182,24 @@ class ZmqConnection(object):
 
         Part of L{IReadDescriptor}.
         """
-        events = self.socket.getsockopt(constants.EVENTS)
+        while True:
+            if self.factory is None:  # disconnected
+                return
 
-        if (events & constants.POLLIN) == constants.POLLIN:
-            while True:
-                if self.factory is None:  # disconnected
-                    return
-                try:
-                    message = self._readMultipart()
-                except error.ZMQError as e:
-                    if e.errno == constants.EAGAIN:
-                        break
+            events = self.socket.getsockopt(constants.EVENTS)
 
-                    raise e
+            if (events & constants.POLLIN) != constants.POLLIN:
+                return
 
-                log.callWithLogger(self, self.messageReceived, message)
+            try:
+                message = self._readMultipart()
+            except error.ZMQError as e:
+                if e.errno == constants.EAGAIN:
+                    continue
+
+                raise e
+
+            log.callWithLogger(self, self.messageReceived, message)
 
     def logPrefix(self):
         """
